@@ -22,19 +22,22 @@
 #  
 from __future__ import unicode_literals
 from django.utils.encoding import smart_text, python_2_unicode_compatible
-from django.utils import six
+from django.utils import six, timezone
 from django.db import models
 from django.db.models import Q, get_model
 from django.utils.dates import WEEKDAYS, MONTHS
 from django.utils.translation import ugettext_noop, ugettext_lazy as _
 from django.core.paginator import Paginator, Page, PageNotAnInteger, EmptyPage
 from django.utils.dateparse import parse_datetime, parse_date, parse_time
+from django.utils.dateformat import format as date_format
 from django.template.defaultfilters import slugify
 from datetime import timedelta
 import operator, re
 
 from reportapi import conf
 from reportapi.python_serializer import serialize
+from reportapi.utils import periods
+from reportapi.exceptions import PeriodsError, ObjectFoundError
 
 DEFAULT_SEARCH_FIELDS = getattr(conf, 'DEFAULT_SEARCH_FIELDS',
     (# Основные классы, от которых наследуются другие
@@ -59,6 +62,11 @@ __conditions__ = (ugettext_noop('isnull'), ugettext_noop('empty'),# ugettext_noo
     ugettext_noop('endswith'), ugettext_noop('iendswith')
 )
 
+DB_OPERATORS = {
+        'exact': '=', 'gt': '>', 'gte': '>=', 'lt': '<', 'lte': '<=',
+        'isnull': 'IS NULL', 'range': 'BETWEEN %s AND %s'
+    }
+
 @python_2_unicode_compatible
 class BaseFilter(object):
     required = None
@@ -66,31 +74,62 @@ class BaseFilter(object):
     conditions = ('isnull', 'exact', 'gt', 'gte', 'lt', 'lte', 'range')
     placeholder = None
     verbose_name = None
+    boolean_labels = {'TRUE': _('yes'), 'FALSE': _('no'), 'NONE': _('any')}
+
+    db_operators = DB_OPERATORS
 
     def __str__(self):
         return '%s:%s' % (self.__class__.__name__, self.name)
 
-    def __init__(self, name, required=False, conditions=None):
+    def __init__(self, name, required=False, conditions=None,
+        verbose_name=None, placeholder=None, **kwargs):
+
         if self.required is None:
             self.required = required
+
         if conditions:
             self.conditions = conditions
-        self.name = slugify(name)
-        self.verbose_name = self.verbose_name or _(name)
 
-    def get_value(self, condition, value):
+        if 'default_condition' in kwargs:
+            self.default_condition = kwargs['default_condition']
+
+        if 'default_value' in kwargs:
+            default_value = kwargs['default_value']
+            if callable(default_value):
+                self.default_value = lambda s: default_value()
+            else:
+                self.default_value = default_value
+
+            if not hasattr(self, 'default_condition'):
+                self.default_condition = self.conditions[0] if self.conditions else 'truth'
+
+        self.name = slugify(name)
+
+        if not verbose_name is None:
+            self.verbose_name = verbose_name
+        else:
+            self.verbose_name = self.verbose_name or _(name)
+
+        if not placeholder is None:
+            self.placeholder = placeholder
+
+    def get_value(self, condition, value, request=None):
         return value
 
-    def get_value_range_label(self, condition, value):
+    def get_value_range_label(self, condition, value, request=None):
         if condition == 'range':
-            value = list(self.get_value_label(condition, value))
+            value = list(self.get_value_label(condition, value, request=request))
             return value[0], value[-1] 
         return None
 
-    def get_value_label(self, condition, value):
-        return self.get_value(condition, value)
+    def get_value_label(self, condition, value, request=None):
+        if condition == 'truth':
+            return self.boolean_labels.get(str(value).upper(), _('any'))
+        elif condition in ('isnull', 'empty'):
+            return self.boolean_labels.get(str(bool(value)).upper())
+        return self.get_value(condition, value, request=request)
 
-    def data(self, condition='truth', value=None, inverse=False, **options):
+    def data(self, condition=ugettext_noop('truth'), value=None, inverse=False, request=None, **options):
         """
         Метод получения информации об установленном фильтре.
         """
@@ -100,23 +139,24 @@ class BaseFilter(object):
         options['inverse'] = inverse
         options['condition'] = condition
         options['condition_label'] = _(condition)
-        options['value'] = self.get_value(condition, value)
-        options['value_label'] = self.get_value_label(condition, value)
+        options['value'] = self.get_value(condition, value, request=request)
+        options['value_label'] = self.get_value_label(condition, value, request=request)
         if condition == 'range':
-            options['value_range_label'] = self.get_value_range_label(condition, value)
+            options['value_range_label'] = self.get_value_range_label(condition, value, request=request)
 
         return options
 
     def condition_list(self):
         return [(x, _(x)) for x in self.conditions or ()]
 
-    def serialize(self):
+    def serialize(self, request=None, **kwargs):
         D = {
             'required': self.required or False,
             'name': self.name,
             'label': self.verbose_name,
             'type': self._type,
             'conditions': self.condition_list(),
+            'boolean_labels': self.boolean_labels,
         }
 
         if self.placeholder:
@@ -124,10 +164,35 @@ class BaseFilter(object):
 
         if hasattr(self, 'options'):
             o = self.options
-            D['options'] = o() if callable(o) else o
+            if callable(o):
+                D['options'] = o(request=request)
+            else:
+                D['options'] = o
 
-        if hasattr(self, 'withseconds'):
-            D['withseconds'] = self.withseconds
+        if hasattr(self, 'get_default_value'):
+            # для получения доступных пользователю объектов передаём request
+            try:
+                D['value'] = self.get_default_value(request=request)
+            except:
+                pass
+            # для datetime, date, time
+            if hasattr(self, 'get_default_server_value'):
+                try:
+                    D['server_value'] = self.get_default_server_value()
+                except:
+                    pass
+
+            if hasattr(self, 'default_condition'):
+                D['condition'] = self.default_condition
+
+        elif hasattr(self, 'default_value'):
+            if callable(self.default_value):
+                D['value'] = self.default_value(request=request)
+            else:
+                D['value'] = self.default_value
+
+            if hasattr(self, 'default_condition'):
+                D['condition'] = self.default_condition
 
         if hasattr(self, 'search_on_date'):
             D['search_on_date'] = self.search_on_date
@@ -138,6 +203,24 @@ class BaseFilter(object):
         if hasattr(self, 'unicode_key'):
             D['unicode_key'] = self.unicode_key
 
+        if hasattr(self, 'usenone'):
+            D['usenone'] = self.usenone
+
+        if hasattr(self, 'format'):
+            D['format'] = self.format
+
+        if hasattr(self, 'formatDate'):
+            D['formatDate'] = self.formatDate
+
+        if hasattr(self, 'formatTime'):
+            D['formatTime'] = self.formatTime
+
+        if hasattr(self, 'use_tz'):
+            D['use_tz'] = self.use_tz
+
+        if hasattr(self, 'use_mask'):
+            D['use_mask'] = self.use_mask
+
         return self.name, D
 
 class FilterObject(BaseFilter):
@@ -159,7 +242,7 @@ class FilterObject(BaseFilter):
         'app.model.objects'
         так и готовым экземпляром менеджера модели
         """
-        super(FilterObject, self).__init__(name, **kwargs)
+        super(FilterObject, self).__init__(name=name, **kwargs)
 
         manager = manager or getattr(self, 'manager', None)
         model = model or getattr(self, 'model', None)
@@ -222,13 +305,23 @@ class FilterObject(BaseFilter):
 
     @property
     def objects(self):
+        """
+        Возвращает все данные из менеждера модели
+        """
         return self.manager.all()
 
-    @property
-    def options(self):
-        return serialize(self.manager.all()[:self.max_options],
+    def options(self, request=None, **kwargs):
+        return serialize(self.get_queryset(request=request, **kwargs)[:self.max_options],
             attrs=self.fields_search, unicode_key=self.unicode_key)
 
+    def get_queryset(self, request=None, **kwargs):
+        """
+        Этот метод может быть переопределён в наследуемых классах,
+        например, если требуется получить определённый набор данных в
+        зависимости от запроса.
+        """
+        return self.objects
+
     def get_paginator(self, queryset, per_page=25, orphans=0,
         allow_empty_first_page=True, **kwargs):
         return Paginator(
@@ -257,27 +350,38 @@ class FilterObject(BaseFilter):
             page_queryset = paginator.page(paginator.num_pages)
         return page_queryset
 
-    def search(self, query, page=1):
+    def search(self, query, page=1, request=None):
         """
         Регистронезависимый поиск по строковым полям, либо по полям с датой
         """
+        qs = self.get_queryset(request=request)
         if self.search_on_date:
-            qs = _date_search_in_fields(self.objects, self.fields_search, query)
+            qs = _date_search_in_fields(qs, self.fields_search, query)
         else:
-            qs = _search_in_fields(self.objects, self.fields_search, query)
+            qs = _search_in_fields(qs, self.fields_search, query)
         page_queryset = self.get_page_queryset(qs, page=page)
         return serialize(page_queryset, attrs=self.fields_search,
             unicode_key=self.unicode_key)
 
-    def get_value(self, condition, value):
+    def get_value(self, condition, value, request=None):
+        qs = self.get_queryset(request=request)
         if condition == 'isnull':
-            return value
+            return bool(value)
         elif condition == 'exact':
-            return self.objects.get(pk=value)
+            try:
+                return qs.get(pk=value)
+            except:
+                raise ObjectFoundError()
         elif condition == 'range':
-            return self.objects.filter(pk__gte=min(value), pk__lte=max(value)).order_by('pk')
+            qs = qs.filter(pk__in=value).order_by('pk')
+            if len(qs) != 2:
+                raise ObjectFoundError()
+            return qs[0], qs[1]
         else:
-            return self.objects.filter(pk__in=list(value)).order_by('pk')
+            qs = qs.filter(pk__in=list(value)).order_by('pk')
+            if not qs:
+                raise ObjectFoundError()
+            return qs
 
 class FilterText(BaseFilter):
     _type = 'text'
@@ -288,11 +392,87 @@ class FilterNumber(BaseFilter):
     _type = 'number'
     conditions = ('exact', 'gt', 'gte', 'lt', 'lte')
 
-class FilterDateTime(BaseFilter):
+# Translated date and time formats
+DATE_FORMAT         = _('Y-m-d')
+TIME_FORMAT         = _('H:i')
+TIME_FORMAT_SEC     = _('H:i:s')
+DATETIME_FORMAT     = _('Y-m-d H:i')
+DATETIME_FORMAT_SEC = _('Y-m-d H:i:s')
+
+class BaseDateTime(BaseFilter):
+    use_mask = True
+    default_condition = 'exact'
+
+    def time_replace(self, value):
+        def _replace(v):
+            v = timezone.localtime(v)
+            if hasattr(v, 'second'):
+                v = v.replace(microsecond=0)
+                if not getattr(self, 'withseconds', False):
+                    v = v.replace(second=0)
+            return v
+
+        if isinstance(value, (list, tuple)):
+            return [_replace(x) for x in value]
+
+        return _replace(value)
+
+    def get_default_value(self, **kwargs):
+        if callable(self.default_value):
+            value = self.default_value()
+        else:
+            value = self.default_value
+
+        value = self.time_replace(value)
+
+        cond = getattr(self, 'default_condition', None)
+
+        if cond and cond == 'range':
+            if not isinstance(value, (list, tuple)):
+                value = date_format(value, '%s' % self.format)
+                return (value, value)
+
+        return date_format(value, '%s' % self.format)
+
+    def get_default_server_value(self, **kwargs):
+        if callable(self.default_value):
+            value = self.default_value()
+        else:
+            value = self.default_value
+
+        value = self.time_replace(value)
+
+        cond = getattr(self, 'default_condition', None)
+
+        if cond and cond == 'range':
+            if not isinstance(value, (list, tuple)):
+                return (value, value)
+            else:
+                return value
+        return value
+
+class FilterDateTime(BaseDateTime):
     _type = 'datetime'
     withseconds = False
+    use_tz = conf.settings.USE_TZ
 
-    def get_value(self, condition, value):
+    default_value = lambda s: timezone.now()
+
+    formatDate = DATE_FORMAT
+
+    @property
+    def format(self):
+        if self.withseconds:
+            return DATETIME_FORMAT_SEC
+        return DATETIME_FORMAT
+
+    @property
+    def formatTime(self):
+        if self.withseconds:
+            return TIME_FORMAT_SEC
+        return TIME_FORMAT
+
+    def get_value(self, condition, value, request=None):
         if isinstance(value, six.string_types):
             value = parse_datetime(value)
         elif isinstance(value, (list, tuple)):
@@ -307,10 +487,12 @@ class FilterDateTime(BaseFilter):
             value = [min(value), max(value)]
         return value
 
-class FilterDate(BaseFilter):
+class FilterDate(BaseDateTime):
     _type = 'date'
+    format = DATE_FORMAT
+    default_value = lambda s: timezone.localtime(timezone.now()).date()
 
-    def get_value(self, condition, value):
+    def get_value(self, condition, value, request=None):
         if isinstance(value, six.string_types):
             value = parse_date(value)
         elif isinstance(value, (list, tuple)):
@@ -325,11 +507,18 @@ class FilterDate(BaseFilter):
             value = [min(value), max(value)]
         return value
 
-class FilterTime(BaseFilter):
+class FilterTime(BaseDateTime):
     _type = 'time'
     withseconds = False
+    default_value = lambda s: timezone.localtime(timezone.now()).time()
 
-    def get_value(self, condition, value):
+    @property
+    def format(self):
+        if self.withseconds:
+            return TIME_FORMAT_SEC
+        return TIME_FORMAT
+
+    def get_value(self, condition, value, request=None):
         if isinstance(value, six.string_types):
             value = parse_time(value)
         elif isinstance(value, (list, tuple)):
@@ -347,11 +536,17 @@ class FilterTime(BaseFilter):
 class FilterBoolean(BaseFilter):
     _type = 'boolean'
     conditions = None
+    usenone = True
+    default_value = None
+    default_condition = 'truth'
 
-    def serialize(self):
-        name, dic = super(FilterBoolean, self).serialize()
-        dic['YESNO'] = _('yes'), _('no')
-        return name, dic
+    def __init__(self, name, usenone=None, **kwargs):
+        super(FilterBoolean, self).__init__(name=name, **kwargs)
+
+        if not usenone is None:
+            self.usenone = usenone
+            if self.default_value is None:
+                self.required = True
 
 class FilterChoice(BaseFilter):
     _type = 'choice'
@@ -365,20 +560,20 @@ class FilterChoice(BaseFilter):
             self._options = dict(options)
         elif not hasattr(self, '_options'):
             self._options = {}
-        super(FilterChoice, self).__init__(name, **kwargs)
+        super(FilterChoice, self).__init__(name=name, **kwargs)
 
     @property
     def options(self):
         return [ {'value':x,'label':y} for x,y in self._options.items() ]
 
-    def get_value_label(self, condition, value):
+    def get_value_label(self, condition, value, request=None):
         dic = self._options
         if condition in ('range', 'in'):
             return [ dic[self.keytype(x)] for x in set(list(value)) if self.keytype(x) in dic ]
         else:
             return dic.get(self.keytype(value), None)
 
-    def get_value(self, condition, value):
+    def get_value(self, condition, value, request=None):
         dic = self._options
         if condition in ('range', 'in'):
             value = [ self.keytype(x) for x in set(list(value)) if self.keytype(x) in dic ]
@@ -434,13 +629,32 @@ PERIODS = {
         'previous_year': _('Previous year'),
     }
 
+# test
+for x in PERIODS.keys():
+    if not hasattr(periods, x):
+        raise PeriodsError(_('Function `%s` not found in reportapi.utils.periods') % x)
+
 class FilterPeriod(FilterChoice):
     _type = 'period'
     placeholder = _('Select period')
     conditions = ('exact',)
     keytype = str
     _options = PERIODS
+    withtime = True
 
+    def get_value_label(self, condition, value, request=None):
+        return self._options.get(self.keytype(value), None)
+
+    def get_value(self, condition, value, request=None):
+        """
+        Если периоды втроенные, то для них есть функции получения
+        объектов datetime. Если нет, то программист должен сам
+        обработать это значение при формировании отчёта.
+        """
+        if value in PERIODS:
+            f = getattr(periods, value)
+            return f(withtime=self.withtime)
+        return value
 
 def _search_in_fields(queryset, fields, query):
     """ Фильтрация """
@@ -464,7 +678,7 @@ def _search_in_fields(queryset, fields, query):
 
     return queryset
 
-def _date_search_in_fields(queryset, fields, query):
+def _date_search_in_fields(queryset, fields, query, format=None):
     """ Фильтрация по дате"""
     if fields:
         query = query.strip()
