@@ -39,22 +39,12 @@
 ////////////////////////////////////////////////////////////////////////
 //                   КОНСТАНТЫ И ГЛОБАЛЬНЫЕ ПЕРЕМЕННЫЕ                //
 ////////////////////////////////////////////////////////////////////////
-var TIMEOUT_PROGRESS = 1000,
-    SERVER_TZ_OFFSET = window.SERVER_TZ_OFFSET || -180; // Europe/Moscow
-
-// Глобальные хранилища-регистраторы
-window.TEMPLATES = {}; // Шаблоны
+window.TEMPLATES = {}; // Скомпилированные шаблоны underscore
 
 ////////////////////////////////////////////////////////////////////////
 //                            НАСТРОЙКИ                               //
 ////////////////////////////////////////////////////////////////////////
 
-/* Настройки шаблонизатора underscore.js в стиле Django */
-//~ _.templateSettings = {
-    //~ interpolate: /\{\{(.+?)\}\}/g,
-    //~ evaluate: /\{\%(.+?)\%\}/g, 
-//~ };
-
 /* Включение Underscore.string методов в пространство имён Underscore */
 _.mixin(_.str.exports());
 
@@ -78,6 +68,7 @@ function handlerHideAlert() {
     $('.alert').alert('close');
     $('#alert-place').css('z-index', '-1000');
 };
+
 function handlerShowAlert(msg, type, callback, timeout) {
     if (DEBUG) {console.log('function:'+'handlerShowAlert')};
     timeout = timeout || 5000;
@@ -144,7 +135,7 @@ function jsonAPI(args, callback, to_console, sync, timeout) {
             handlerShowAlert(json.message, 'alert-danger');
             clearInterval(window.REPORT.process);
             window.REPORT.process = undefined;
-            $('.progress .progress-bar').removeClass('progress-striped active');
+            $('.progress .progress-bar').removeClass('progress-bar-striped active');
             handlerSetProgress(0);
         }
         /* При нормальном возврате в debug-режиме выводим в консоль
@@ -154,7 +145,6 @@ function jsonAPI(args, callback, to_console, sync, timeout) {
             if (DEBUG) {
                 var o = new Object;
                 $.extend(true, o, json.data);
-                //~ console.log(o);
                 console.log($.toJSON(json.message));
             };
             return callback(json, status, xhr);
@@ -170,15 +160,18 @@ function jsonAPI(args, callback, to_console, sync, timeout) {
 /* Обработчик создания отчёта */
 function handlerCreateReport(force) {
     if (DEBUG) {console.log('function:'+'handlerCreateReport')};
-    $('.action-create-report, .action-recreate-report, .action-preview, .action-remove')
+    $('.action-create-report, .action-recreate-report, .action-preview, .action-download, .action-remove')
         .attr('disabled', 'disabled');
     $('#view-document-place').hide();
 
     window.REPORT.id = null;
     window.REPORT.process_filters = {};
     $.each(window.REPORT.filters, function(key, item) {
-        if (item.condition || item.value != null) {
-            window.REPORT.process_filters[item.name] = {condition:item.condition, value:item.value};
+        if (item.value != null && item.value != undefined) {
+            window.REPORT.process_filters[item.name] = {
+                condition: item.condition,
+                value: (item.server_value != undefined) ? item.server_value : item.value,
+            };
         };
     });
     var args = {
@@ -194,18 +187,6 @@ function handlerCreateReport(force) {
     return jqxhr;
 };
 
-/* Обработчик события создания отчёта */
-function eventCreateReport(event) {
-    if (DEBUG) {console.log('function:'+'eventCreateReport')};
-    handlerCreateReport();
-};
-
-/* Обработчик события пересоздания отчёта */
-function eventRecreateReport(event) {
-    if (DEBUG) {console.log('function:'+'eventRecreateReport')};
-    handlerCreateReport(true);
-};
-
 /* Обработчик запуска ожидания создания отчёта */
 function handlerCheckProcess() {
     if (DEBUG) {console.log('function:'+'handlerCheckProcess')};
@@ -241,6 +222,9 @@ function handlerCheckProcess() {
                 $('.action-preview')
                     .removeClass('btn-warning').addClass('btn-default')
                     .find('.fa').removeClass('fa-bug').addClass('fa-search');
+                $('.action-download')
+                    .attr('href', json.data.url)
+                    .prop("disabled", false).removeAttr('disabled');
             };
             // If user can remove this report
             if (json.data.has_remove) {
@@ -253,8 +237,7 @@ function handlerCheckProcess() {
             };
 
             // TODO: animation or 1-2 seconds after
-            $pbar.removeClass('progress-striped active');
-            //~ handlerSetProgress(0);
+            $pbar.removeClass('progress-bar-striped active');
     
             $('.action-recreate-report').prop("disabled", false).removeAttr('disabled');
 
@@ -266,10 +249,9 @@ function handlerCheckProcess() {
             };
         } else if (now >= max){
             // report not ready and count request to server >= maximum request
-            $pbar.addClass('progress-striped active');
+            $pbar.addClass('progress-bar-striped active');
         } else {
             // report not ready
-            console.log(TIMEOUT_PROGRESS + now);
             handlerSetProgress(TIMEOUT_PROGRESS + now);
         };
     };
@@ -302,19 +284,36 @@ function handlerRemoveDocument(id, refresh) {
             success = function() {
                 $('#div-document-'+id).remove();
                 $('#view-document-place').hide();
-                handlerAfterChanges();
-                if (refresh) { window.location.reload() };
+                if (refresh) window.location.reload()
+                else handlerAfterChanges();
             };
         new jsonAPI(args, success);
     };
     
 };
 
+// События
+
+/* Обработчик события создания отчёта */
+function eventCreateReport(event) {
+    if (DEBUG) {console.log('function:'+'eventCreateReport')};
+    handlerCreateReport();
+    event.preventDefault();
+};
+
+/* Обработчик события пересоздания отчёта */
+function eventRecreateReport(event) {
+    if (DEBUG) {console.log('function:'+'eventRecreateReport')};
+    handlerCreateReport(true);
+    event.preventDefault();
+};
+
 /* Обработчик события удаления отчёта */
 function eventRemoveDocument(event) {
     if (DEBUG) {console.log('function:'+'eventRemoveDocument')};
     var id = null;
     handlerRemoveDocument(id);
+    event.preventDefault();
 };
 
 ////////////////////////////////////////////////////////////////////////
@@ -322,9 +321,11 @@ function eventRemoveDocument(event) {
 ////////////////////////////////////////////////////////////////////////
 
 /* Обработчик установки поиска-выбора для фильтров объектов */
-function handlerSetSelectizers($box) {
+function handlerSetSelectizers(filter) {
     if (DEBUG) {console.log('function:'+'handlerSetSelectizers')};
-    var $box = $box || $('body');
+    var $box = $('#valuebox-'+filter.name);
+
+    if ($box.size() <1) return false;
 
     $.each($box.find('select[data-type="object"]'), function(index, select) {
 
@@ -359,17 +360,16 @@ function handlerSetSelectizers($box) {
                             'method': "reportapi.object_search",
                             'section': window.REPORT.section || null,
                             'name': window.REPORT.name,
-                            'filter_name': filter_name,
+                            'filter_name': filter.name,
                             'query': query,
                         },
                         success = function(res) {
-                            //~ console.log(res);
                             callback(res.data.object_list);
                         };
                     new jsonAPI(args, success);
                 },
                 onChange: function(value) {
-                    if (filter.condition == 'range' && value.length != 2) {
+                    if (filter.condition == 'range' && (!value || value.length != 2)) {
                         filter.value = null;
                     } else {
                         filter.value = (value == '') ? null : value;
@@ -400,7 +400,7 @@ function handlerSetSelectizers($box) {
                 create: false,
                 maxItems: (filter.condition == 'range') ? 2 : undefined,
                 onChange: function(value) {
-                    if (filter.condition == 'range' && value.length != 2) {
+                    if (filter.condition == 'range' && (!value || value.length != 2)) {
                         filter.value = null;
                     } else {
                         filter.value = (value == '') ? null : value;
@@ -414,32 +414,83 @@ function handlerSetSelectizers($box) {
             });
         }
     });
+
+    return true;
 };
 
-/* Обработчик установки масок для input */
-function handlerMaskInputs($box) {
-    if (DEBUG) {console.log('function:'+'handlerMaskInputs')};
-    var $box = $box || $('body');
+/* Обработчик установки datetimepicker для input */
+function handlerSetDatetimePickers(filter) {
+    if (DEBUG) {console.log('function:'+'handlerSetDatetimePickers')};
+    if (!filter || !(filter.type in {'datetime':'','date':'','time':''})) return false;
+    var args = {
+            lang: LANGUAGE_CODE,
+            format: filter.format,
+            timepicker: true,
+            datepicker: true,
+            mask: filter.use_mask,
+        };
 
-    // Инициализация маски для дат
-    $.mask.definitions['1']='[0-1]';
-    $.mask.definitions['2']='[0-2]';
-    $.mask.definitions['3']='[0-3]';
-    $.mask.definitions['5']='[0-5]';
-    $.each($box.find('[data-mask]'), function(i, item) {
-        var $item = $(item),
-            data = $(item).data();
-        $item.mask(data.mask, {
-            completed: function() {
-                var filter = REPORT.filters[this.context.name];
-                filter.value = this.val();
-                if (filter.condition == 'range') {
-                    filter.value = filter.value.split(RANGE_SPLIT);
-                };
-                if (DEBUG) {console.log('function:'+'handlerMaskInputs:completed; filter=' + filter.value)};
+    if (filter.type == 'datetime') {
+        args.formatDate = filter.formatDate;
+        args.formatTime = filter.formatTime;
+        args.onChangeDateTime = function(dt, $input) {
+            /* сохраняем время так, как будто браузер находится в той-же
+             * временной зоне, что и сервер.
+             */
+            var val;
+            if (filter.use_tz) {
+                dt.setMinutes(dt.getMinutes() - (dt.getTimezoneOffset() - SERVER_TZ_OFFSET));
+                val = dt.toISOString();
+            } else {
+                val = ''+dt.getFullYear()+'-'+(dt.getMonth()+1)+'-'+dt.getDate()
+                     +'T'+dt.getHours()+':'+dt.getMinutes()+':'+dt.getSeconds()
+                     +'.'+dt.getMilliseconds();
             }
-        });
-    });
+            $input.data('server_value', val);
+        }
+    } else if (filter.type == 'time') {
+        args.formatTime = filter.format;
+        args.datepicker = false;
+        args.onChangeDateTime = function(dt, $input) {
+            /* сохраняем время в понятном серверу формате.*/
+            $input.data('server_value',
+                ''+dt.getHours()+':'+dt.getMinutes()+':'+dt.getSeconds()
+                +'.'+dt.getMilliseconds());
+        }
+    } else {
+        args.formatDate = filter.format;
+        args.timepicker = false;
+        args.onChangeDateTime = function(dt, $input) {
+            /* сохраняем дату в понятном серверу формате.*/
+            dt.setMinutes(dt.getMinutes() - (dt.getTimezoneOffset() - SERVER_TZ_OFFSET));
+            $input.data('server_value',
+                ''+dt.getFullYear()+'-'+(dt.getMonth()+1)+'-'+dt.getDate());
+        }
+    };
+
+    if (filter.condition == 'range') {
+        var args1 = {}, args2 = {},
+            $input1 = $('#value-'+ filter.name +'-range1'),
+            $input2 = $('#value-'+ filter.name +'-range2');
+
+        $.extend(true, args1, args);
+        $.extend(true, args2, args);
+
+        // TODO: сделать ограничение для datetime
+        if (filter.type == 'date') {
+            args1.onShow = function(ct){
+                this.setOptions({ maxDate: $input2.data().server_value ? $input2.val() : false })
+            };
+            args2.onShow = function(ct){
+                this.setOptions({ minDate: $input1.data().server_value ? $input1.val() : false })
+            }
+        }
+
+        $input1.datetimepicker(args1);
+        $input2.datetimepicker(args2);
+    } else {
+        $('#value-'+ filter.name).datetimepicker(args);
+    }
 
     return true;
 };
@@ -450,9 +501,14 @@ function handlerCheckRequiredValue() {
 
     var completed = true;
     $.each(REPORT.filters, function(key, item) {
-        if ((item.required) && (item.value === null || item.value === undefined)) {
-            completed = false;
-        };
+        if (item.required) {
+            if (item.value === null || item.value === undefined) {
+                completed = false;
+            } else if (item.condition == 'range' &&
+                (item.value.length < 2 || item.value[0] === null || item.value[1] === null)) {
+                completed = false;
+            };
+        }
     });
     if (completed) {
         $('.action-create-report').prop("disabled", false).removeAttr("disabled");
@@ -471,57 +527,143 @@ function handlerAfterChanges() {
         window.REPORT.process = null;
     };
     handlerSetProgress(0);
-    $('.action-create-report, .action-preview, .action-remove')
+    $('.action-create-report, .action-preview, .action-download, .action-remove')
         .attr('disabled', 'disabled');
     $('.action-recreate-report:visible').hide();
     $('.action-create-report:hidden').show();
     handlerCheckRequiredValue();
 };
 
-// События
 
+// События
 
 /* Обработчик события изменения значения фильтра */
 function eventChangeValue(event) {
     if (DEBUG) {console.log('function:'+'eventChangeValue')};//, event)};
-    var value = $(event.target).val(),
-        filter = REPORT.filters[event.target.name];
-    if (event.target.type == 'checkbox') {
-        filter.value = event.target.checked;
-    } else if ($(event.target).attr('data-mask') !== undefined) {
-        filter.value = filter.value || null;
+    var value = this.value,
+        data = $(this).data(),
+        server_value = data.server_value,
+        filter = REPORT.filters[this.name],
+        test_datetime = function(objects) {
+            var test = true;
+            $.each(objects, function(i, v){
+                if (!v || v.indexOf('_') >= 0) { test = false }
+            })
+            return test 
+        };
+
+    if (filter.type == 'boolean') $('#valuebox-'+filter.name).html('');
+
+    if (this.type == 'radio') {
+        // Для радио-кнопок с булевыми значениями
+        filter.value = (value == 'true') ? true : (value == 'false') ? false : null;
+        if (server_value != undefined) filter.server_value = filter.value;
+    } else if (filter.condition == 'range' && filter.type in {'datetime':'','date':'','time':''}) {
+        // Для datetimepickers с условием диапазона
+        var other, other_value;
+        if (this.id.match(/.*-range1/g)) {
+            other = $('#value-'+ filter.name +'-range2');
+            other_value = other.val();
+            filter.value = test_datetime([other_value, value]) ? [value, other_value] : null;
+            filter.server_value = test_datetime([other_value, value]) ? [server_value, other.data().server_value] : null;
+        } else {
+            other = $('#value-'+ filter.name +'-range1');
+            other_value = other.val();
+            filter.value = test_datetime([other_value, value]) ? [other_value, value] : null;
+            filter.server_value = test_datetime([other_value, value]) ? [other.data().server_value, server_value] : null;
+        }
     } else {
         filter.value = value || null;
+        if (server_value != undefined) filter.server_value = server_value;
     };
+
     handlerAfterChanges();
 };
 
-/* Обработчик события изменения условия фильтра */
+/* Обработчик события изменения условия фильтра
+ * 
+ * TODO: сделать установку дефолтных значений для даты-времени
+ * 
+ */
 function eventConditionChange(event) {
     if (DEBUG) {console.log('function:'+'eventConditionChange')};//, event)};
 
-    var filter_name = $(event.target).data()['name'],
-        filter = REPORT.filters[filter_name];
-    filter.condition = event.target.value || null;
-    //~ console.log(filter.condition);
-    if (filter.condition in {'isnull':0, 'empty':0}) {
-        filter.value = false;
-    } else if (!filter.condition || filter.condition in {'range':0,'in':0} || filter.type == 'object') {
+    var filter = REPORT.filters[$(this).data()['name']],
+        prev_condition = filter.condition,
+        prev_value     = filter.value,
+        prev_server_value = filter.server_value;
+
+    filter.condition = this.value || null;
+
+
+    if (!filter.condition) {
+        // Сброс условия
         filter.value = null;
+        if (filter.server_value != undefined) filter.server_value = null;
+
+    } else if (filter.condition in {'isnull':0, 'empty':0}) {
+        // Для условия "пусто" значение по-умолчанию истинно
+        filter.value = true;
+
+    } else if (prev_value === true || prev_value === false || prev_value === null || prev_value === undefined) {
+        // Предыдущего значения небыло или стояло одно из условий: 'isnull', 'empty'
+        filter.value = null;
+        if (filter.server_value != undefined) filter.server_value = null;
+
+    } else if (filter.type in {'object':0, 'choice':0, 'month':0, 'weekday':0, 'period':0}) {
+        // Объекты и списки выбора сбрасываются потому, что не найдено
+        // решение их установки в библиотеке selectize
+        filter.value = null;
+
+    } else if (filter.condition in {'range':0,'in':0}) {
+        // Для этих условий заполняем одним значением
+        if (!(prev_condition in {'range':0,'in':0})) {
+
+            filter.value = (filter.condition == 'range') ?
+                    [prev_value, prev_value] : [prev_value];
+
+            if (filter.server_value != undefined) {
+                filter.server_value = (filter.condition == 'range') ?
+                    [prev_server_value, prev_server_value] : [prev_server_value];
+            }
+
+        } else if (prev_condition == 'in' && filter.condition == 'range') {
+
+            filter.value = prev_value.slice(0, 2);
+
+            if (filter.value.length < 2) { filter.value[1] = filter.value[0] }
+
+            if (filter.server_value != undefined) {
+
+                filter.server_value = prev_server_value.slice(0, 2);
+
+                if (filter.server_value.length < 2) { filter.server_value[1] = filter.server_value[0] }
+            }
+        } /* else {  prev_condition == 'range' && filter.condition == 'in'  } */
     } else {
-        filter.value = $('#value-'+filter_name).val() || null;
+        // Любые другие условия
+        if (prev_condition in {'range':0,'in':0}) {
+
+            filter.value = prev_value ? prev_value[0] : null;
+
+            if (filter.server_value != undefined) {
+                filter.server_value = prev_server_value ? prev_server_value[0] : null;
+            }
+        }
     };
+
     var html = TEMPLATES.filter({ data: filter });
-    $('#valuebox-'+filter_name).html(html);
-    handlerSetSelectizers($('#valuebox-'+filter_name));
-    handlerMaskInputs($('#valuebox-'+filter_name));
+
+    $('#valuebox-'+filter.name).html(html);
+
+    handlerSetSelectizers(filter);
+    handlerSetDatetimePickers(filter);
 
     handlerAfterChanges();
 
     return true;
 };
 
-
 /* Проверка события нажатия клавиши управления браузером */
 function keyDownIsControl(event) {
     if (DEBUG) {console.log('function:'+'keyDownIsControl')};//, event)};
@@ -535,34 +677,12 @@ function keyDownIsControl(event) {
     return false;
 };
 
-/* Обработчик события нажатия клавиши на поле ввода дат и времени */
-function eventKeyDownOnDateTime(event) {
-    if (DEBUG) {console.log('function:'+'eventKeyDownOnDateTime')};//, event)};
-
-    var prepare = function(event) {
-        value = event.target.value || '';
-        //~ console.log(value, event.charCode);
-    };
-
-    // Enabled keys browser control
-    if (keyDownIsControl(event)) { return true }
-    // Replace valide values
-    else if (event.which >= 48 && event.which <= 57) { // 0-9
-        prepare(event);
-        return true;
-    }
-
-    return false;
-};
-
-
 /* Обработчик события нажатия клавиши на поле ввода чисел */
 function eventKeyDownOnNumber(event) {
-    //~ if (DEBUG) {console.log('function:'+'eventKeyDownOnNumber')};//, event)};
+    //if (DEBUG) {console.log('function:'+'eventKeyDownOnNumber')};//, event)};
 
     var adddot = function(event) {
         value = event.target.value || '';
-        //~ console.log('search', value.search('.'));
         if (value.length < 1) {
             event.target.value = '0.';
         }
@@ -584,6 +704,7 @@ function eventKeyDownOnNumber(event) {
     return false;
 };
 
+
 ////////////////////////////////////////////////////////////////////////
 //                              ПРОЧЕЕ                                //
 ////////////////////////////////////////////////////////////////////////
@@ -591,9 +712,9 @@ function eventKeyDownOnNumber(event) {
 /* Обработчик прогрессбара */
 function handlerSetProgress(now) {
     if (DEBUG) {console.log('function:'+'handlerSetProgress')};
+    now = now || 0;
     var $pbar = $('.progress .progress-bar'),
         max = Number($pbar.attr('aria-valuemax')) || 100,
-        now = now || 0,
         width = (now / max) * 100;
     width = width > 100 ? 100 : width;
     $pbar.attr('aria-valuenow', now).css('width', width +'%');
@@ -619,26 +740,6 @@ function handlerShowDocument(href, name, inwindow) {
     }
 };
 
-/* Обработчик отображения дополнительных фильтров */
-function handlerShowAdditionalFilters() {
-    if (DEBUG) {console.log('function:'+'handlerShowAdditionalFilters')};
-    $('.additional-filter').show();
-    $('.action-hide-additional').show();
-    $('.action-show-additional').hide();
-
-    return true;
-};
-
-/* Обработчик сокрытия дополнительных фильтров */
-function handlerHideAdditionalFilters() {
-    if (DEBUG) {console.log('function:'+'handlerHideAdditionalFilters')};
-    $('.additional-filter').hide();
-    $('.action-hide-additional').hide();
-    $('.action-show-additional').show();
-    return true;
-};
-
-
 /* Обработчик установки шаблонов */
 function handlerTemplates() {
     if (DEBUG) {console.log('function:'+'handlerTemplates')};
@@ -660,15 +761,10 @@ function handlerBindinds() {
     $('body').on('change', 'select[id^="condition-"]', eventConditionChange);
     $('body').on('keydown', 'input[data-validate="number"]', eventKeyDownOnNumber);
     $('body').on('change', 'input[id^="value-"]', eventChangeValue);
-    $('body').on('click', '.action-hide-additional', handlerHideAdditionalFilters);
-    $('body').on('click', '.action-show-additional', handlerShowAdditionalFilters);
+
     $('body').on('click', '.action-create-report', eventCreateReport);
     $('body').on('click', '.action-recreate-report', eventRecreateReport);
 
-    // Биндинги на отчёты
-    //~ $('body').on('click', '[data-action=object_print]',   eventObjectPrint);
-    //~ $('body').on('click', '[data-action=object_print]',   eventObjectPrint);
-
     return true;
 };
 
@@ -680,6 +776,9 @@ function handlerBindinds() {
 $(document).ready(function($) {
     if (DEBUG) {console.log('function:'+'$(document).ready')};
 
+    // Инициализация moment.js
+    moment.locale(LANGUAGE_CODE);
+
     // Инициализация шаблонов Underscore
     handlerTemplates();
 
@@ -690,8 +789,4 @@ $(document).ready(function($) {
     // Установка биндингов для элементов
     handlerBindinds();
 
-    // Прочее
-    handlerMaskInputs();
-    handlerHideAdditionalFilters();
-
 });
