@@ -45,7 +45,8 @@ urlpatterns = patterns('reportapi.views',
     url(r'^$', 'index', name='index'),
     url(r'^api/', 'api', name='api'),
     url(r'^list/(?P<section>\w+)/$', 'report_list', name='report_list'),
-    url(r'^docs/(?P<pk>\d+)/$', 'get_document', name='get_document'),
+    url(r'^doc/(?P<pk>\d+)/$', 'get_document', name='get_document'),
+    url(r'^doc/(?P<pk>\d+)/download/$', 'get_document', name='download_document', kwargs={'download': True}),
     url(r'^docs/$', 'documents', name='documents'),
     url(r'^docs/(?P<section>\w+)/$', 'documents', name='documents_section'),
     url(r'^docs/(?P<section>\w+)/(?P<name>\w+)/$', 'documents', name='documents_section_name'),
